//***********************************************************************
//
//  Paint1.cpp
//
//***********************************************************************

#include <afxwin.h>
#include "Resource.h"
#include "Paint1.h"

CMyApp myApp;

/////////////////////////////////////////////////////////////////////////
// CMyApp member functions

BOOL CMyApp::InitInstance ()
{
    m_pMainWnd = new CMainWindow;
    m_pMainWnd->ShowWindow (m_nCmdShow);
    m_pMainWnd->UpdateWindow ();
    return TRUE;
}

/////////////////////////////////////////////////////////////////////////
// CMainWindow message map and member functions

BEGIN_MESSAGE_MAP (CMainWindow, CFrameWnd)
    ON_WM_PAINT ()
    ON_COMMAND (IDM_FILE_NEW, OnFileNew)
    ON_COMMAND (IDM_FILE_EXIT, OnFileExit)
    ON_COMMAND_RANGE (IDM_WIDTH_VTHIN, IDM_WIDTH_VTHICK, OnWidth)
    ON_COMMAND_RANGE (IDM_COLOR_BLACK, IDM_COLOR_WHITE, OnColor)
    ON_UPDATE_COMMAND_UI (IDM_FILE_NEW, OnUpdateFileNewUI)
    ON_UPDATE_COMMAND_UI_RANGE (IDM_WIDTH_VTHIN, IDM_WIDTH_VTHICK,
        OnUpdateWidthUI)
    ON_UPDATE_COMMAND_UI_RANGE (IDM_COLOR_BLACK, IDM_COLOR_WHITE,
        OnUpdateColorUI)
    ON_WM_LBUTTONDOWN ()
    ON_WM_MOUSEMOVE ()
    ON_WM_LBUTTONUP ()
END_MESSAGE_MAP ()

CMainWindow::CMainWindow ()
{
    m_nColor = IDM_COLOR_RED;
    m_nWidth = IDM_WIDTH_MEDIUM;
    m_lineArray.SetSize (0, 64);

    CString strWndClass = AfxRegisterWndClass (
        0,
        myApp.LoadStandardCursor (IDC_CROSS),
        (HBRUSH) (COLOR_WINDOW + 1),
        myApp.LoadIcon (IDR_MAINFRAME)
    );
    
    Create (strWndClass, "Paint1", WS_OVERLAPPEDWINDOW,
        rectDefault, NULL, MAKEINTRESOURCE (IDR_MAINFRAME));

    LoadAccelTable (MAKEINTRESOURCE (IDR_MAINFRAME));
}

CMainWindow::~CMainWindow ()
{
    DeleteAllLines ();
}

void CMainWindow::OnPaint ()
{
    CPaintDC dc (this);
    int nCount = m_lineArray.GetSize ();

    if (nCount) {
        for (int i=0; i<nCount; i++)
            ((CLine*) m_lineArray[i])->Draw (&dc);
    }
}

void CMainWindow::OnFileNew ()
{
    DeleteAllLines ();
    Invalidate ();
}

void CMainWindow::OnUpdateFileNewUI (CCmdUI* pCmdUI)
{
    pCmdUI->Enable (m_lineArray.GetSize ());
}

void CMainWindow::OnFileExit ()
{
    SendMessage (WM_CLOSE, 0, 0);
}

void CMainWindow::OnWidth (UINT nID)
{
    m_nWidth = nID;
}

void CMainWindow::OnColor (UINT nID)
{
    m_nColor = nID;
}

void CMainWindow::OnUpdateWidthUI (CCmdUI* pCmdUI)
{
    pCmdUI->SetCheck (pCmdUI->m_nID == m_nWidth);
}

void CMainWindow::OnUpdateColorUI (CCmdUI* pCmdUI)
{
    pCmdUI->SetCheck (pCmdUI->m_nID == m_nColor);
}

void CMainWindow::OnLButtonDown (UINT nFlags, CPoint point)
{
    m_ptFrom = point;
    m_ptTo = point;
    SetCapture ();
}

void CMainWindow::OnMouseMove (UINT nFlags, CPoint point)
{
    if (GetCapture () == this) {
        CClientDC dc (this);
        InvertLine (&dc, m_ptFrom, m_ptTo);
        InvertLine (&dc, m_ptFrom, point);
        m_ptTo = point;
    }
}

void CMainWindow::OnLButtonUp (UINT nFlags, CPoint point)
{
    static COLORREF crColors[8] = {
        RGB (  0,   0,   0),    // Black
        RGB (  0,   0, 255),    // Blue
        RGB (  0, 255,   0),    // Green
        RGB (  0, 255, 255),    // Cyan
        RGB (255,   0,   0),    // Red
        RGB (255,   0, 255),    // Magenta
        RGB (255, 255,   0),    // Yellow
        RGB (255, 255, 255)     // White
    };

    static UINT nWidths[5] = { 1, 8, 16, 24, 32 };

    if (GetCapture () == this) {
        ReleaseCapture ();

        CClientDC dc (this);
        InvertLine (&dc, m_ptFrom, m_ptTo);
        CLine* pLine = NULL;

        try {
            pLine = new CLine (m_ptFrom, m_ptTo,
                nWidths[m_nWidth - IDM_WIDTH_VTHIN],
                crColors[m_nColor - IDM_COLOR_BLACK]);

            m_lineArray.Add (pLine);
            pLine->Draw (&dc);
        }
        catch (CMemoryException* e) {
            MessageBox ("Out of memory. You must clear the " \
                "drawing area before adding more lines.", "Error",
                MB_ICONEXCLAMATION | MB_OK);

            if (pLine != NULL)
                delete pLine;
            e->Delete ();   
        }       
    }
}

void CMainWindow::InvertLine (CDC* pDC, CPoint ptFrom, CPoint ptTo)
{
    int nOldMode = pDC->SetROP2 (R2_NOT);

    pDC->MoveTo (ptFrom);
    pDC->LineTo (ptTo);

    pDC->SetROP2 (nOldMode);
}

void CMainWindow::DeleteAllLines ()
{
    int nCount = m_lineArray.GetSize ();

    for (int i=0; i<nCount; i++)
        delete m_lineArray[i];

    m_lineArray.RemoveAll ();
}

/////////////////////////////////////////////////////////////////////////
// CLine member functions

CLine::CLine (CPoint ptFrom, CPoint ptTo, UINT nWidth, COLORREF crColor)
{
    m_ptFrom = ptFrom;
    m_ptTo = ptTo;
    m_nWidth = nWidth;
    m_crColor = crColor;
}

void CLine::Draw (CDC* pDC)
{
    CPen pen (PS_SOLID, m_nWidth, m_crColor);

    CPen* pOldPen = pDC->SelectObject (&pen);
    pDC->MoveTo (m_ptFrom);
    pDC->LineTo (m_ptTo);

    pDC->SelectObject (pOldPen);
}
